/* ************************************************************************** */
/*                                       _____           _                    */
/*  libg1m/format/addin.h               |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/11/02 14:15:46                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#ifndef LIBG1M_FORMAT_ADDIN_H
# define LIBG1M_FORMAT_ADDIN_H
# include <stdint.h>
# pragma pack(1)

/*	Add-ins are compiled programs. They only have one G1M part,
 *	These add-ins usually have the `g1a` extension.
 *
 *	with one subheader, which is the following: */

# define G1A_ICON_WIDTH 30
# define G1A_ICON_HEIGHT 17
struct g1a_subheader {
	/* the internal name, of format "@APPNAME".
	 *	useful for add-ins calling themselves... I guess? */
	uint8_t internal_name[8];

	/* the number of estrips (I don't know yet) */
	uint8_t estrips_count;

	/* the add-in version, of format "01.23.4567"
	 *	the "01.23" will be displayed in SYSTEM > VERSION */
	uint8_t version[10];

	/* the add-in creation type, of format "YYYY.MMDD.HHMM" */
	uint8_t creation_date[14];

	/* 30x17 pixel menu icon bitmap */
	uint8_t icon[68];

	/* program title */
	uint8_t title[8];

	/* and the filesize of the part! */
	uint32_t filesize;
};

/*	Then the G1A file will just contain the add-in code and stop.
 *
 *	There is a declination for fx-CG calculators, which has the `g3a` extension.
 *	Here's its header: */

# define G3A_ICON_WIDTH 92
# define G3A_ICON_HEIGHT 64
struct g3a_subheader {
	/* byte checksum: 0x0000 to 0x001F (standard header)
	 * and 0x0024 to EOF (everything after this checksum), minus 4. */
	uint32_t checksum;

	/* magic sequence: {0x01, 0x01} */
	uint8_t magic[2];

	/* undocumented */
	uint8_t undocumented[8];

	/* control: filesize - 0x7000 - 4 */
	uint32_t control;

	/* undocumented */
	uint8_t undocumented2[14];

	/* title (zero terminated) */
	uint8_t title[16];

	/* undocumented */
	uint8_t undocumented3[12];

	/* filesize */
	uint32_t filesize;

	/* internal name (preceded by @, filled up with zeros */
	uint8_t internal_name[11];

	/* language labels */
	uint8_t language_1_label[24];
	uint8_t language_2_label[24];
	uint8_t language_3_label[24];
	uint8_t language_4_label[24];
	uint8_t language_5_label[24];
	uint8_t language_6_label[24];
	uint8_t language_7_label[24];
	uint8_t language_8_label[24];

	/* eAct strip flag (0x00: cannot be used, 0x01: can be used) */
	uint8_t eact_strip_flag;

	/* some null bytes */
	uint8_t zeroes[4];

	/* version: "01.00.0000" then zeroes */
	uint8_t version[12];

	/* timestamp: "2012.0903.1652" */
	uint8_t stamp[14];

	/* null bytes, again */
	uint8_t zeroes2[38];

	/* eAct strip labels */
	uint8_t eact_strip_label_1[36];
	uint8_t eact_strip_label_2[36];
	uint8_t eact_strip_label_3[36];
	uint8_t eact_strip_label_4[36];
	uint8_t eact_strip_label_5[36];
	uint8_t eact_strip_label_6[36];
	uint8_t eact_strip_label_7[36];
	uint8_t eact_strip_label_8[36];

	/* eAct icon */
	uint8_t icon[0x300];

	/* unused */
	uint8_t unused[0x92c];

	/* G3A filename (seriously?!) */
	uint8_t g3a_filename[0x144];

	/* selected and unselected icon image */
	uint8_t selected_icon_image[0x3000];
	uint8_t unselected_icon_image[0x3000];
};

# pragma pack()
#endif /* LIBG1M_FORMAT_ADDIN_H */
