/* ************************************************************************** */
/*                                       _____           _                    */
/*  fontcharacter/character.c           |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/11/21 13:29:30                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#include <libg1m/fontcharacter.h>
#include <string.h>

/**
 *	g1m_mbtofc:
 *	Multi-byte to FONTCHARACTER.
 *
 *	Based on what is said in the fx-9860G SDK.
 *	Let's hope it stays true.
 *
 *	Based on how `mbtowc` works.
 *
 *	@arg	pfc		pointer to the FONTCHARACTER character.
 *	@arg	s		the multi-byte source string.
 *	@arg	n		the size of the source buffer.
 *	@return			the number of bytes used (-1 if error).
 */

int g1m_mbtofc(FONTCHARACTER *pfc, const char *s, size_t n)
{
	/* null string? */
	if (!s)
		return (2);

	/* no space? */
	if (!n)
		return (-1);

	/* extended char? */
	if (strchr("\x7F\xF7\xF9\xE5\x56\x57", *s))
	{
		if (n < 2) return (-1);
		*pfc = (*s << 8) + *(s + 1);
		return (2);
	}

	/* single-byte character */
	*pfc = *s;
	return (*s != 0);
}

/**
 *	g1m_fctomb:
 *	FONTCHARACTER to multi-byte.
 *
 *	Based on the same document, and on how `wctomb` works.
 *
 *	Do you think a battery has self-consciousness?
 *
 *	@arg	s		pointer to the multi-byte string.
 *	@arg	fc		the FONTCHARACTER character.
 *	@return			the
 */

int g1m_fctomb(char *s, FONTCHARACTER fc)
{
	/* no string? */
	if (!s)
		return (2);

	/* extended? */
	if (fc > 0xff) {
		*s++ = fc >> 8;
		*s = fc & 0xff;
		return (2);
	}

	/* normal char. */
	*s = fc;
	return (fc != 0);
}
