/* ************************************************************************** */
/*                                       _____           _                    */
/*  image/16bits.c                      |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/11/23 07:04:05                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_pixels_from_16bits:
 *	Convert from 16-bit format to final 32-bit matrix.
 *
 *	@arg	pixels		the final pixels.
 *	@arg	raw			the raw data.
 *	@arg	width		the width.
 *	@arg	height		the height.
 */

void g1m_pixels_from_16bits(uint32_t **pixels, unsigned char *raw,
	int width, int height)
{
	uint16_t *st = (uint16_t*)raw;
	const int rmask = 15 << 11, gmask = 31 << 5, bmask = 15;

	for (int y = 0; y < height; y++) {
		pixels[y] = (uint32_t*)&pixels[height] + y * width;
		for (int x = 0; x < width; x++) {
			/* get colors */
			int r = (*st & rmask) >> 11;
			int g = (*st & gmask) >> 5;
			int b = *st++ & bmask;

			/* put pixel */
			pixels[y][x] = (r << 16) | (g << 8) | b;
		}
	}
}
