/* ************************************************************************** */
/*                                       _____           _                    */
/*  image/4bits.c                       |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/11/23 07:04:02                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#include <libg1m/internals.h>

/**
 *	g1m_pixels_from_4bits:
 *	Convert 4-bits formatted image to the final 32-bit matrix.
 *
 *	@arg	pixels		the final pixels.
 *	@arg	raw			the raw data.
 *	@arg	width		the width.
 *	@arg	height		the height.
 */

void g1m_pixels_from_4bits(uint32_t **pixels, unsigned char *raw,
	int width, int height)
{
	int mask = 0xf0;
	int pixnum = 0;

	for (int y = 0; y < height; y++) {
		pixels[y] = (uint32_t*)&pixels[height] + y * width;
		for (int x = 0; x < width; x++) {
			/* get byte using max and iterate (i love trying to optimize) */
			int b = (*raw & mask) >> (pixnum & 4);
			raw += (pixnum & 8) >> 4;
			pixnum += 4;

			/* put pixel */
			b *= 16; /* extend from 4 bits to 8 */
			pixels[y][x] = b | (b << 8) | (b << 16); /* grayyy */
		}
	}
}
