/* ************************************************************************** */
/*                                       _____           _                    */
/*  user/free.c                         |_   _|__  _   _| |__   ___ _   _     */
/*  | Project: libg1m                     | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/11/21 09:13:10                               |___/     */
/*                                                                            */
/* ************************************************************************** */
#include <libg1m/internals.h>
#include <stdlib.h>

/**
 *	g1m_free_mcsfile:
 *	Free an MCS file handle.
 *
 *	@arg	handle		the handle to free.
 */

void g1m_free_mcsfile(g1m_mcsfile_t *handle)
{
	/* free cells */
	if (handle->type & (g1m_mcstype_mat | g1m_mcstype_list
	  | g1m_mcstype_spreadsheet)
	 && handle->columns && handle->rows) {
		free(handle->cells[0]);
		free(handle->cells);
	}

	/* free the images */
	if (handle->type & (g1m_mcstype_pict | g1m_mcstype_capt)) {
		free(handle->image);
		if (handle->type & g1m_mcstype_capt)
			free(handle->second_image);
	}

	/* free the handle */
	free(handle);
}

/**
 *	g1m_free_mcs:
 *	Free all of the MCS.
 *
 *	@arg	handle		the handle to close.
 */

void g1m_free_mcs(g1m_t *handle)
{
	/* check if mcs */
	if (!handle->mcs)
		return ;

	/* foreach mcs */
	g1m_mcs_t **mcs = handle->mcs;
	int mcs_count = handle->mcs_count;
	for (int i = 0; i < mcs_count; i++) {
		/* check if used */
		if (!mcs[i])
			continue ;

		/* foreach file in mcs */
		g1m_mcsfile_t **files = mcs[i]->files;
		int file_count = mcs[i]->file_count;
		for (int j = 0; j < file_count; j++)
			/* free the file if exists */
			if (files[j]) g1m_free_mcsfile(files[j]);
		free(files);
		free(mcs[i]);
	}
	free(mcs);
	handle->mcs = NULL;
}

/**
 *	g1m_free:
 *	Free handle data.
 *
 *	@arg	handle		the handle to close.
 */

void g1m_free(g1m_t *handle)
{
	if (!handle) return ;

	/* addin time! */
	if (handle->type & g1m_type_addin)
		free(handle->pixels);

	/* mcs time! */
	if (handle->type & g1m_type_mcs)
		g1m_free_mcs(handle);

	/* picture time! */
	if (handle->type & g1m_type_picture)
		free(handle->pixels);

	/* free the handle */
	free(handle);
}
